/*
 * Copyright 2020 Ladifire. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import {AriaLabelingProps, AsyncLoadable, Collection, CollectionChildren, DOMProps, MultipleSelection, Node, SectionProps, Sortable, StyleProps} from '@react-ui-types/shared';
import {Key, ReactElement, ReactNode} from 'react';

export interface TableProps<T> extends MultipleSelection, Sortable {
  children: ReactElement<TableHeaderProps<T> | TableBodyProps<T> | SectionProps<T> | RowProps<T>>[],
  disabledKeys?: Iterable<Key>
}

export interface BorderProperties {
  top?: boolean;
  bottom?: boolean;
  left?: boolean;
  right?: boolean;
}

export interface BorderProps {
  header?: BorderProperties;
  row?: BorderProperties;
  cell?: BorderProperties;
}

export interface TableBordersProps {
  style?: 'all' | 'outer' | 'none' | 'top-left' | BorderProps;
  widths?: {
    outer?: number;
    inner?: number;
  };
  color?: string;
  className?: string;
}

export interface SpectrumTableProps<T> extends TableProps<T>, DOMProps, AriaLabelingProps, StyleProps {
  density?: 'compact' | 'regular' | 'spacious',
  overflowMode?: 'wrap' | 'truncate',
  isQuiet?: boolean,
  fullWidth?: boolean,
  width?: number,
  noBorders?: boolean,
  noHeader?: boolean,
  renderEmptyState?: () => JSX.Element,
  borders?: TableBordersProps,
  className?: string,
  headerClassName?: string,
  bodyClassName?: string,
  rowClassName?: string,
}

export interface TableHeaderProps<T> {
  columns?: T[],
  children: ColumnElement<T> | ColumnElement<T>[] | ColumnRenderer<T>
}

type ColumnElement<T> = ReactElement<ColumnProps<T>>;
type ColumnRenderer<T> = (item: T) => ColumnElement<T>;
export interface ColumnProps<T> {
  title?: ReactNode,
  children: ReactNode | ColumnElement<T> | ColumnElement<T>[],
  childColumns?: T[],
  'aria-label'?: string,
  width?: number | string,
  minWidth?: number | string,
  maxWidth?: number | string,
  defaultWidth?: number | string,
  allowsSorting?: boolean,
  allowsResize?: boolean,
}

// TODO: how to support these in CollectionBuilder...
export interface SpectrumColumnProps<T> extends ColumnProps<T> {
  align?: 'start' | 'center' | 'end',
  allowsResizing?: boolean,
  allowsReordering?: boolean,
  allowsSorting?: boolean,
  isSticky?: boolean, // shouldStick??
  isRowHeader?: boolean,
  showDivider?: boolean,
  sortComponent?: ReactNode,
}

export interface TableBodyProps<T> extends AsyncLoadable {
  children: CollectionChildren<T>,
  items?: Iterable<T>
}

export interface RowProps<T> {
  UNSAFE_className?: string,
  // treeble case?
  childItems?: Iterable<T>,
  hasChildItems?: boolean,
  children: CellElement | CellElement[] | CellRenderer,
  textValue?: string, // ???
  'aria-label'?: string // ???
}

export interface CellProps {
  children: ReactNode,
  textValue?: string,
  'aria-label'?: string
}

export type CellElement = ReactElement<CellProps>;
export type CellRenderer = (columnKey: Key) => CellElement;

export interface TableCollection<T> extends Collection<TableNode<T>> {
  headerRows: TableNode<T>[],
  columns: TableNode<T>[],
  columnKeys: Key[],
  rowHeaderColumnKeys: Set<Key>,
  body: TableNode<T>
}

export interface TableNode<T> extends Node<T> {
  column?: TableNode<T>,
  colspan?: number
}
